// james - LCD.H - LCDT.C header file
#ifndef INC_LCD_H
#define INC_LCD_H

#include "types.h"

// #include <f4x5.h>    // small size numbers font 4 x 5 pixel
#include "lcdt/F5x8.h"    // standard font 5 x 6 and 5 x 8 pixel
//#include "lcdt/F10x12.h"  // middle size numbers font 10 x 12 pixel
// #include <f15x18.h>  // large size numbers 15 x 18 pixel


#define LCD_BUSPIN1 16 	/* Bus on Pins P1.16->P1.23 */

#define LCD_DATA    (uint32_t)(0xff<<LCD_BUSPIN1)
#define LCD_DATABUS	0x00FF0000

#define LCD_RST		(uint32_t)(1<<28)  /* Reset on P1.28 */
#define LCD_CE		(uint32_t)(1<<26)  /* Chip enable on P1.26  */
#define LCD_WR		(uint32_t)(1<<24)  /* Write on P1.24 */
#define LCD_CD		(uint32_t)(1<<27)  /* Data/Instr. P1.27 */
#define LCD_RD		(uint32_t)(1<<25)  /* Read  on P1.25  */

#define LCD_CONTROL	(uint32_t)(LCD_RD|LCD_CE|LCD_WR|LCD_CD|LCD_RST)



// #define  LCD         0x0F000          // base address
// #define  LCD_dato    LCD + 0x0        // C/D = 0
// #define  LCD_cmd     LCD + 0x1	      // C/D = 1

// LCD status word masks
#define  ready_to_read          0x03
#define  ready_to_write         0x03
#define  ready_to_autowrite     0x08
#define  ready_to_autoread      0x04

// command codes
#define  text_home_address      0x40
#define  text_area              0x41
#define  graphic_home_address   0x42
#define  graphic_area           0x43
#define  cursor_pointer         0x21
#define  offset_register        0x22
#define  address_pointer        0x24
#define  autowrite_ON           0xB0
#define  autoread_ON            0xB1
#define  auto_OFF               0xB2
#define  display_OFF            0x90
#define  blink_ON               0x91
#define  cursor_ON              0x92
#define  text_ON                0x94
#define  graphic_ON             0x98
#define  cursor_pattern         0xA0
#define  data_read              0xC5
#define  data_write             0xC4
#define  write_incr_adr         0xC0

// start address of display RAM (graphic mode
#define  GHOME   0

#define  Ymax  128  //128        			// max number of vertical pixels
#define  Xmax  240  //128       			// max number of horizzontal pixels
#define  Nmax  (Xmax/8)  			// max number of coloumns of 8 pixels
#define  Cmax  (Xmax/8)  			// max number of characters for row

// display model
#define  OEMDISP                0x80
#define  DATAINT                0

// font codes
// #define  F15x18                 0x10
#define  F10x12                 0x20
#define  F5x8                   0x30
// #define  F5x6                   0x40
// #define  F4x5                   0x50

// display mode
#define  NORMAL                 0     // black on white
#define  REVERSE                8     // white on black

// four valid number of LCD pages. From 0 to 3

// display zones
#define  UP_LEFT     1
#define  UP_RIGHT    2
#define  LOW_LEFT    4
#define  LOW_RIGHT   8
#define  LEFT       (UP_LEFT + LOW_LEFT)
#define  RIGHT      (UP_RIGHT + LOW_RIGHT)
#define  UPPER      (UP_LEFT + UP_RIGHT)
#define  LOWER      (LOW_LEFT + LOW_RIGHT)
#define  WIDE       (LEFT + RIGHT)
#define  ALL         WIDE


// external calls of LCDT library
extern void  refresh_LCD        (uint8_t page);
extern void  gotoxy             (uint8_t x, uint8_t y);
extern void  set_display        (uint8_t font, uint8_t mode);
extern void  wrbm               (uint8_t w, uint8_t h, uint8_t * bm);
extern void  wrch               (uint8_t ch);
extern void  wrstr              (char* s);
extern uint8_t ch_width           (void);
extern uint8_t dp_width           (void);
extern uint8_t ch_height          (void);
extern void  clearfield         (uint8_t w, uint8_t h);
extern void  clearto            (uint8_t x, uint8_t y);
extern void  set_pixel          (uint8_t x, uint8_t y);
extern void  reset_pixel        (uint8_t x, uint8_t y);
extern void  hline              (int dx);
extern void  vline              (int dy);
extern void  box		        (uint8_t dx, uint8_t dy);
extern void  clearlcd           (uint8_t zona);
extern void  goto_page          (uint8_t page);
extern void  set_page           (uint8_t page);
extern void  copy_page          (uint8_t src_page, uint8_t dest_page);
extern void  save_current_page  (uint8_t dest_page);
//extern void  init_LCD           (uint8_t tipo);
extern void cfgGLCD(void);
uint8_t lcd_autoread(void);
#endif
