/******************************************************************/
/* james - LCD.C library - T6963C display driver 64 x 128 pixels  */
/******************************************************************/
//#include <c_ext.h>
//#include <absacc.h>
//#include <intrins.h>

#include <string.h>

#include "types.h"
#include "lpc21xx.h"
#include "lcd.h"     // usefull type codes font, display modes, address etc.
#include "delay.h"
const uint8_t mask[] = { 0, 0x80, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC, 0xFE, 0xFF };

// LCDT variables
// uint8_t idata xStart;        // horizzontal position 0..Xmax-1
// uint8_t idata yStart;        // vertical position 0..Ymax-1
// uint8_t idata status;    // page - mode - font
uint8_t xStart;        // horizzontal position 0..Xmax-1
uint8_t yStart;        // vertical position 0..Ymax-1
uint8_t status;    // page - mode - font
// #define  Ymax  128  //128        			// max number of vertical pixels
// #define  Xmax  240  //128       			// max number of horizzontal pixels
// #define  Nmax  (Xmax/8)  			// max number of coloumns of 8 pixels
// #define  Cmax  (Xmax/6)  			// max number of characters for row
// LCD 4 pages buffer + 1 row
//uint8_t framebuffer [3 * Ymax * Nmax + Nmax];
uint8_t framebuffer[Ymax * Nmax + Nmax];
// font pointers
const uint8_t *p5x8 [256];

// bit status mask
#define  TMASK   0x80             // display model mask
#define  FMASK   0x70             // font mask
#define  MMASK   0x08             // mode mask
#define  PMASK   0x07             // page mask

#define  display_model  (status & TMASK)
#define  display_mode   (status & MMASK)
#define  selected_font  (status & FMASK)
#define  selected_page  (status & PMASK)

#define  DY  ((display_model == OEMDISP) ? 1 : 0)

//uint8_t rotatebits(uint8_t bits, uint8_t cnt){
//
// asm("mov %0, %1, ror "c" (cnt)" : "=r" (result) : "r" (value));
//
//}

/* Low level lcd access */
/********************************************
 * Function name: lcd_readstat           *
 * Description:   Read Status From LCD      *
 *******************************************/

static uint8_t lcd_readstat(void)
{
  char lcdstatus;
  IO1DIR |= (LCD_DATABUS);
  IO1CLR = (LCD_DATABUS); /* all low */
  IO1DIR &= ~(LCD_DATABUS);
  IO1SET= LCD_WR;						/* make sure the WR/ line is high */
  IO1SET= LCD_CD;						/* clr the CD line low to data */
  IO1CLR= LCD_CE;						/* enable chip select */
  IO1CLR= LCD_RD;						/* set the RD/ line low */
//	glcd_delay(1);
  IO1CLR= LCD_RD; 					/* extra read delay */
  lcdstatus=(unsigned char)(IO1PIN>>LCD_BUSPIN1);/* fetch in the data */
  IO1SET= LCD_RD;						/* set the RD/ line back high */
  IO1SET= LCD_CE;						/* force the CE back high */
  return(lcdstatus);
}
/********************************************
 * Function name: glcd_busy_wait           *
 * Description:   wait for LCD to be ready      *
 *******************************************/
static void lcd_busychk(void)
{
  char lcdstat;
  do
    {
      lcdstat=lcd_readstat();
    }
  while ((0x03 & lcdstat) != 0x03); /* wait till STA1=1 && STA0=1 */
}

/********************************************
 * Function name: lcd_sendbyte            *
 * Description:   Write Data to LCD         *
 *******************************************/
static void lcd_writebyte(uint8_t dat)
{
  IO1DIR |= (LCD_DATABUS);
  IO1CLR = (uint32_t)(LCD_DATABUS); /* all low */
  IO1SET=LCD_RD;						/* make sure the RD/ line is high */
  IO1CLR= LCD_CD;						/* clr the CD line low to data */
  IO1SET = (uint32_t)(dat<<LCD_BUSPIN1);
  IO1CLR=LCD_CE;						/* enable chip select */
  IO1CLR=LCD_WR;						/* set the WR/ line low */
  IO1CLR=LCD_WR;						/* extra delay for pulse width */
  IO1SET=LCD_WR;						/* set the WR/ line back high */
  IO1SET= LCD_CE;						/* force the CE back high */
  IO1CLR = (uint32_t)(LCD_DATABUS); /* all low */
}
/********************************************
 * Function name: lcd_readbyte             *
 * Description:   Read Data From LCD        *
 *******************************************/
static uint8_t lcd_readbyte(void)
{
  char dat;
  IO1DIR |= (LCD_DATABUS);
  IO1CLR = (uint32_t)(LCD_DATABUS); /* all low */
  IO1SET= LCD_WR;						/* make sure the WR/ line is high */
  IO1CLR= LCD_CD;						/* clr the CD line low to data */
  IO1CLR= LCD_RD;						/* set the RD/ line low */
  IO1CLR= LCD_CE;						/* enable chip select */
// DelayUS(1);
  dat=(uint8_t)(IO1PIN>>LCD_BUSPIN1);	/* fetch in the data */
  IO1SET= LCD_CE;						/* force the CE back high */
  IO1SET= LCD_RD;						/* set the RD/ line back high */
  return(dat);

}
/********************************************
** Check for autowrite status if 1 send data to write
** Busy check is not used in auto mode
 *******************************************/
static void lcd_autowrite(uint8_t byte)
{
  char lcdstat;
  do
    {
      lcdstat=lcd_readstat();
    }
  while ((0x08 & lcdstat) != 0x08);		/* wait till STA3=1 */

  lcd_writebyte(byte);

}
/********************************************
** Check for autoread status if 1 read and return  data
** Busy check is not used in auto mode
 *******************************************/
uint8_t lcd_autoread(void)
{
  uint8_t dat;
  uint8_t lcdstat;
  do
    {
      lcdstat=lcd_readstat();
    }
  while ((0x04 & lcdstat) != 0x04);		/* wait till STA2=1 */
  dat = lcd_readbyte();
  return(dat);
}
/********************************************
 * Function name: lcd_sendbytes         *
 * Description:   Send data to write      *
 *******************************************/
static void lcd_sendbyte(uint8_t byte)
{
  lcd_busychk();
  lcd_writebyte(byte);
}

/********************************************
 * Function name: lcd_sendcmd         *
 * Description:   Write Command to LCD      *
 *******************************************/
static void lcd_sendcmd(uint8_t dat)
{
  lcd_busychk();
  IO1DIR |= (LCD_DATABUS);
  IO1CLR = (LCD_DATABUS); /* all low */
  IO1SET=LCD_RD;						/* make sure the RD/ line is high */
  IO1SET= LCD_CD;						/* clr the CD line low to data */
  IO1SET = (uint32_t)(dat<<LCD_BUSPIN1);
  IO1CLR=LCD_CE;						/* enable chip select */
  IO1CLR=LCD_WR;						/* set the WR/ line low */
//	glcd_delay(1);
  IO1CLR=LCD_WR;						/* extra delay for pulse width */
  IO1SET=LCD_WR;						/* set the WR/ line back high */
  IO1SET= LCD_CE;						/* force the CE back high */
  IO1CLR = (LCD_DATABUS); /* all low */

}
/*******************************************
 * Function name: lcd_setaddr              *
 * Description:   Send address then command*
 *******************************************/
static void lcd_setaddr( uint8_t cmd, uint16_t addr)
{
  uint8_t c;
  c = addr&0x00FF;
  lcd_sendbyte(c);		//Send LSB of 'Addr' first
  c = addr >> 8;
  lcd_sendbyte(c);		//Send MSB of 'Addr' afterwards
  lcd_sendcmd(cmd);
}
/************************************************************************************************************/
/* Frame Buffer Routines starts here  */
/************************************************************************************************************/
/*******************************************
  Rotate bits right number of places
*******************************************/
static uint8_t rotr(uint8_t invalue, uint8_t places)
{
  uint8_t outvalue;
  /*  First a normal shift  */
  outvalue = invalue << (places % 8);
  /*  Then place the part that's shifted off the screen at the end  */
  outvalue |= invalue >> (8  - (places % 8));
  return outvalue;
}



static uint32_t current_addr_pointer (void)
/**************************************************************/
{
  return 1024 * selected_page + yStart*Nmax + xStart/8;
}


//void lcd_sendcmd (uint8_t cmd)
///**************************************************************/
//{
//  while (ready_to_write != (ready_to_write & XBYTE[LCD_cmd]));
//  XBYTE[LCD_cmd] = cmd;
//}


//void send_dato (uint8_t dato)
///**************************************************************/
//{
//  while (ready_to_write != (XBYTE[LCD_cmd] & ready_to_write));
//  XBYTE[LCD_dato] = dato;
//}


// void auto_write (uint8_t dato)
// /**************************************************************/
// {
  //while ((XBYTE[LCD_cmd] & ready_to_autowrite) == 0);
  //XBYTE[LCD_dato] = dato;
  // while((lcd_readstat() & ready_to_autowrite) == 0);
  // lcd_sendbyte(dato);
// }


//void send_cmd2 (uint8_t cmd, uint32_t dato)
///**************************************************************/
//{
//  lcd_sendbyte(byte_basso(dato));
//  lcd_sendbyte(byte_alto(dato));
//  lcd_sendcmd(cmd);
//}


void gotoxy (uint8_t x, uint8_t y)
/**************************************************************/
{
  xStart = x;
  yStart = y;
}


void set_display (uint8_t font, uint8_t mode)
/**************************************************************/
{
  status = (status & ~(MMASK|FMASK)) | (mode & MMASK) | (font & FMASK);
}

/**************************************************************/
/* writes character in the display zone limited by 			  */
/* box (xStart, yStart, xStart+w, yStart+h)	and bm sets the font              */
/**************************************************************/
void wrbm (uint8_t w, uint8_t h, unsigned char *bm)
{
  static uint8_t i, j, k, a, b, c, d, n, m0, m1, ncbm;
  uint32_t aa, bb;
  ncbm = 1 + (w-1) / 8;        	// bm[] build ncbm * h bytes
  k = xStart + w - 1;
  m0 = mask[xStart % 8];             // first coloumn mask
  m1 = mask[1 + (k % 8)];        // last coloumn mask
  n = 1 + k/8 - xStart/8;            // max num of coloumns
  aa = current_addr_pointer();

// clipping if h and n are greather than Ymax and Nmax
  if (h > Ymax - yStart) h = Ymax - yStart;
  if (n > Nmax - xStart/8) n = Nmax - xStart/8;

// 1 pixel rows loop
  i = 0;
  do
    {
      // set pixel without modify xStart and yStart
      bb = aa + i*Nmax;
      // 8 pixels coloumns loop
      k = 0;
      while (k < n)
        {
          if (k) b = c;
          j = ncbm*i + k;
//          c = _cror_(bm[j], xStart % 8);
          c = rotr(bm[j], xStart);//xStart % 8);/*                              */

          a = c;
          if (display_mode == REVERSE)
            {
              a = ~a;
              b = ~b;
            }
          a &= ~m0;
          // current contents of the first and last coloumns have
          // to be restored -> read and save
          d = framebuffer[bb];
          // writes, recovering if required what has been saved
          framebuffer[bb++] =
            // just one coloumn...
            (n == 1) ? (a & m1) | (d & (m0|~m1)) :
            // first coloumn of 2 or more...
            (k == 0) ? a | (d & m0) :
            // last coloumn of 2 more...
            (k == n-1) ?
            (ncbm == k) ? (b & m1) | (d & (~m1)) :
            (b & m0) | (a & m1) | (d & (~m1)) :
                // intermediat coloumn of 3 or more...
                a | (b & m0);
          k++;
        }
    }
  while (++i < h);

// set pixel for next writing if required
  xStart = (xStart + w < Xmax) ? xStart + w : Xmax;
}


void wrch (uint8_t ch)
/*************************************************************/
{
  switch (selected_font)
    {
    // case F4x5:
      // wrbm((ch < 10) ? 8 : 5, 5, bm4x5(ch));
      // break;
    // case F5x6:
      // wrbm((ch < 10) ? 13 : 6, 6, p5x8[ch] + 1);
      // break;
    case F5x8:
      wrbm((ch < 10) ? 13 : 6, 8, p5x8[ch]);
      break;
//    case F10x12:
//      wrbm((ch == '.') ? 5 : (ch < 10) ? 24 : 12, 12, bm10x12(ch));
//      break;
    // case F15x18:
      // wrbm((ch == '.') ? 7 : 18, 18, bm15x18(ch));
      // break;
    }
}


uint8_t ch_width ()
/*************************************************************/
{
  switch (selected_font)
    {
    // case F4x5:
      // return 5;
    // case F5x6:
    case F5x8:
      return 6;
//    case F10x12:
//      return 12;
    // case F15x18:
      // return 18;
    }
    return 0;
}


uint8_t dp_width ()
/*************************************************************/
{
  switch (selected_font)
    {
    // case F4x5:
      // return 5;
    // case F5x6:
    case F5x8:
      return 6;
//    case F10x12:
//      return 5;
    // case F15x18:
      // return 7;
    }return 0;
}


uint8_t ch_height ()
/*************************************************************/
{
  switch (selected_font)
    {
    // case F4x5:
      // return 5;
    // case F5x6:
      // return 6;
    case F5x8:
      return 8;
//    case F10x12:
//      return 12;
    // case F15x18:
      // return 18;
    }
    return 0;
}


void wrstr (char* s)
/*************************************************************/
{
  while (*s) wrch(*s++);
}

/**************************************************************/
/* erase all the contents in the box (xStart, yStart, xStart+w, yStart+h)     */
/* and when finished set graphical cursor to home (xStart, yStart)    */
/**************************************************************/
void clearfield (uint8_t w, uint8_t h)

{
  const uint8_t blank[] = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 };
  uint8_t xs = xStart;
  uint8_t ys = yStart;
  while (h--)
    {
      xStart = xs;
      yStart = ys + h;
      wrbm(w, 1, blank);
    }
  xStart = xs;
  yStart = ys;
}

/**************************************************************/
/* erase all the contents in the box (xStart, yStart, x, y)           */
/* and then set cursor to right							      */
/**************************************************************/
void clearto (uint8_t x, uint8_t y)
{
  clearfield(x - xStart, y - yStart);
  xStart = x;
}


void set_pixel (uint8_t x, uint8_t y)
/**************************************************************/
{
  xStart = x;
  yStart = y;
  framebuffer[current_addr_pointer()] |= 1 << (7&(~x));
}



void hline (int dx)
/**************************************************************/
/* draw an horizzontal line of |dx|+1 pixels starting from    */
/* the current home position (xStart,yStart).                         */
/* New curren position = end of line				          */
/**************************************************************/
{
  status ^= MMASK;
  if (dx >=0)
    {
      clearfield(1+(uint8_t)dx, 1);
      gotoxy(xStart+(uint8_t)dx, yStart);
    }
  else
    {
      dx = -dx;
      gotoxy(xStart-(uint8_t)dx, yStart);
      clearfield(1+(uint8_t)dx, 1);
    }
  status ^= MMASK;
}


void vline (int dy)
/**************************************************************/
/* draw a vertical line of |dy|+1 pixels starting from        */
/* the current home position (xStart,yStart).                         */
/* New curren position = end of line				          */
/**************************************************************/
{
  uint8_t m = 1 << (7&(~xStart));
  uint32_t aa = current_addr_pointer();
  yStart += dy;
  if (dy >= 0) do
      {
        if (status & MMASK) framebuffer[aa] &= ~m;
        else framebuffer[aa] |= m;
        aa += Nmax;
      }
    while (dy--);
  else do
      {
        if (status & MMASK) framebuffer[aa] &= ~m;
        else framebuffer[aa] |= m;
        aa -= Nmax;
      }
    while (dy++);
}



void box (uint8_t dx, uint8_t dy)
/**************************************************************/
/* draw a box with dimension of dx+1 and  dy+1 pixels         */
/* valid values: dx = 0..127, dy = 0..63                      */
/* (dx = dy = 0 is an unefficient way to draw a pixel         */
/**************************************************************/
{
  hline(dx);
  vline(dy);
  hline(-(char)dx);
  vline(-(char)dy);
}



void clearlcd (uint8_t zona)
/**************************************************************/
{
  uint32_t aa;
  uint8_t xin, yin, nx, ny, i;
  xin = (zona & LEFT) ? 0 : 64;
  yin = (zona & UPPER) ? 0 : 32;
  nx = ((zona & RIGHT) && (zona & LEFT)) ? 16 : 8;
  ny = ((zona & UPPER) && (zona & LOWER)) ? 64 : 32;
  while (ny--)
    {
      xStart = xin;
      yStart = yin + ny;
      aa = current_addr_pointer();
      i = nx;
      while (i--) framebuffer[aa+i] = 0;
    }
  xStart = xin;
  yStart = yin;
}


void set_page (uint8_t page)
/**************************************************************/
{
  status = (status & ~PMASK) | (page & PMASK);
}


void goto_page (uint8_t page)
/**************************************************************/
{
  lcd_setaddr(graphic_home_address, GHOME + 1024*(page & PMASK));
}



void copy_page (uint8_t src_page, uint8_t dest_page)
/**************************************************************/
{
  memcpy(&framebuffer[1024*(uint32_t)dest_page], &framebuffer[1024*(uint32_t)src_page], 1024);
}


void refresh_LCD (uint8_t page)
/**************************************************************/
{
  uint32_t i = 1024 * (uint32_t)page;
  uint32_t imax = i + 1024;
  lcd_setaddr(address_pointer, GHOME + i + Nmax*DY);
  lcd_sendcmd(autowrite_ON);
  do
    {
      lcd_autowrite(framebuffer[i]);
    }
  while (++i < imax);
  lcd_sendcmd(auto_OFF);
  goto_page(page);
}


void save_current_page (uint8_t dest_page)
/**************************************************************/
{
  copy_page(status & PMASK, dest_page);
}


static void init_LCD (uint8_t tipo)
/**************************************************************/
{
  uint8_t ch;
  status = tipo;
  lcd_setaddr (graphic_area, 30); 			// 30 coloumns of 8 pixels
  lcd_sendcmd  (0x80);             			// OR mode, internal char gen.
  lcd_sendcmd  (graphic_ON);
  set_display (F5x8, NORMAL);
  set_page(0);
  clearlcd(ALL);
  refresh_LCD(0);
  goto_page(0);

  ch = 255;
  do
    {
      p5x8[ch] = bm5x8(ch);
    }
  while (ch--);
}

/********************************************
 * Function name: initialize                *
 * Description:   initialize Ports & pins   *
 *******************************************/
void cfgGLCD(void)  // make sure control lines are at correct levels
{
  /* Set Control Lines to outputs and High */
  IO1DIR |= LCD_CONTROL;	        // all selected as outputs
  /* initialize the port control lines to the LCD module */
  IO1SET= LCD_CE;					/* set chip select high off output */
  IO1SET= LCD_RD;					/* set RD line high as output */
  IO1SET= LCD_WR;		   			/* set WR line high as output */
  IO1SET= LCD_CD;					/* set the CD line low as output */
  IO1DIR |= (LCD_DATABUS);		/* Databus all output */

  IO1SET= LCD_RST;					/* set RST signal high off output */
  DelayMS(5);
//  lcd_delay(100000); //  _delay_ms(5);
  IO1CLR= LCD_RST;				/* set the reset line low */
//  lcd_delay(100000);//  _delay_ms(5);
  DelayMS(5);
  IO1SET=LCD_RST;				/* release reset to back high */

  // CurGfxPage = GPAGE1;
  // CurTxtPage = TPAGE1;
  init_LCD(DATAINT);					/* initialize glcd */
  // lcd_setup();					/* initialize glcd */
}
